@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,9 +26,12 @@
 #include <optional>
 #include <algorithm>
 #include <functional>
+#include <iostream>
+#include <fstream>
 #include <cstdint>
 #include <cstdio>
 #include <cmath>
+#include <locale>
 
 #include "frozen/string.h"
 #include "frozen/unordered_map.h"
@@ -80,21 +83,22 @@ static gboolean ucl_reply = FALSE;
 static gboolean extended_urls = FALSE;
 static gboolean mime_output = FALSE;
 static gboolean empty_input = FALSE;
-static gboolean compressed = FALSE;
+static gboolean compressed = TRUE;
 static gboolean profile = FALSE;
 static gboolean skip_images = FALSE;
 static gboolean skip_attachments = FALSE;
 static const char *pubkey = nullptr;
 static const char *user_agent = "rspamc";
+static const char *files_list = nullptr;
 
 std::vector<GPid> children;
 static GPatternSpec **exclude_compiled = nullptr;
 static struct rspamd_http_context *http_ctx;
 
-static gint retcode = EXIT_SUCCESS;
+static int retcode = EXIT_SUCCESS;
 
-static gboolean rspamc_password_callback(const gchar *option_name,
-										 const gchar *value,
+static gboolean rspamc_password_callback(const char *option_name,
+										 const char *value,
 										 gpointer data,
 										 GError **error);
 
@@ -176,6 +180,8 @@ static GOptionEntry entries[] =
 		 "Skip attachments when learning/unlearning fuzzy", nullptr},
 		{"user-agent", 'U', 0, G_OPTION_ARG_STRING, &user_agent,
 		 "Use specific User-Agent instead of \"rspamc\"", nullptr},
+		{"files-list", '\0', 0, G_OPTION_ARG_FILENAME, &files_list,
+		 "Read one or more newline separated filenames to scan from file", nullptr},
 		{nullptr, 0, 0, G_OPTION_ARG_NONE, nullptr, nullptr, nullptr}};
 
 static void rspamc_symbols_output(FILE *out, ucl_object_t *obj);
@@ -496,8 +502,8 @@ auto sort_ucl_container_with_default(T &cont, const char *default_sort,
 
 
 static gboolean
-rspamc_password_callback(const gchar *option_name,
-						 const gchar *value,
+rspamc_password_callback(const char *option_name,
+						 const char *value,
 						 gpointer data,
 						 GError **error)
 {
@@ -519,16 +525,13 @@ rspamc_password_callback(const gchar *option_name,
 			else {
 				/* Strip trailing spaces */
 				auto *map = (char *) locked_mmap.value().get_map();
-				auto *end = map + locked_mmap.value().get_size() - 1;
-
-				while (g_ascii_isspace(*end) && end > map) {
-					end--;
-				}
-
-				end++;
-				value_view = std::string_view{map, static_cast<std::size_t>(end - map + 1)};
-				processed_passwd.assign(std::begin(value_view), std::end(value_view));
-				processed_passwd.push_back('\0');
+				value_view = std::string_view{map, locked_mmap->get_size()};
+				auto right = value_view.end() - 1;
+				for (; right > value_view.cbegin() && g_ascii_isspace(*right); --right)
+					;
+				std::string_view str{value_view.begin(), static_cast<size_t>(right - value_view.begin()) + 1};
+				processed_passwd.assign(std::begin(str), std::end(str));
+				processed_passwd.push_back('\0'); /* Null-terminate for C part */
 			}
 		}
 		else {
@@ -558,7 +561,7 @@ rspamc_password_callback(const gchar *option_name,
  * Parse command line
  */
 static void
-read_cmd_line(gint *argc, gchar ***argv)
+read_cmd_line(int *argc, char ***argv)
 {
 	GError *error = nullptr;
 	GOptionContext *context;
@@ -672,7 +675,7 @@ check_rspamc_command(const char *cmd) -> std::optional<rspamc_command>
 static void
 print_commands_list()
 {
-	guint cmd_len = 0;
+	unsigned int cmd_len = 0;
 
 	rspamc_print(stdout, "Rspamc commands summary:\n");
 
@@ -1048,21 +1051,21 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 				std::string colorized_action;
 				switch (act.value()) {
 				case METRIC_ACTION_REJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::red), "reject");
+					colorized_action = fmt::format(fmt::fg(fmt::color::red), "{}", "reject");
 					break;
 				case METRIC_ACTION_NOACTION:
-					colorized_action = fmt::format(fmt::fg(fmt::color::green), "no action");
+					colorized_action = fmt::format(fmt::fg(fmt::color::green), "{}", "no action");
 					break;
 				case METRIC_ACTION_ADD_HEADER:
 				case METRIC_ACTION_REWRITE_SUBJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::orange), ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::fg(fmt::color::orange), "{}", ucl_object_tostring(elt));
 					break;
 				case METRIC_ACTION_GREYLIST:
 				case METRIC_ACTION_SOFT_REJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::gray), ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::fg(fmt::color::gray), "{}", ucl_object_tostring(elt));
 					break;
 				default:
-					colorized_action = fmt::format(fmt::emphasis::bold, ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::emphasis::bold, "{}", ucl_object_tostring(elt));
 					break;
 				}
 
@@ -1584,10 +1587,10 @@ rspamc_output_headers(FILE *out, struct rspamd_http_message *msg)
 
 static void
 rspamc_mime_output(FILE *out, ucl_object_t *result, GString *input,
-				   gdouble time, GError *err)
+				   double time, GError *err)
 {
-	const gchar *action = "no action", *line_end = "\r\n", *p;
-	gdouble score = 0.0, required_score = 0.0;
+	const char *action = "no action", *line_end = "\r\n", *p;
+	double score = 0.0, required_score = 0.0;
 	gboolean is_spam = FALSE;
 	auto nl_type = RSPAMD_TASK_NEWLINES_CRLF;
 
@@ -1748,10 +1751,10 @@ rspamc_mime_output(FILE *out, ucl_object_t *result, GString *input,
 
 static void
 rspamc_client_execute_cmd(const struct rspamc_command &cmd, ucl_object_t *result,
-						  GString *input, gdouble time, GError *err)
+						  GString *input, double time, GError *err)
 {
-	gchar **eargv;
-	gint eargc, infd, outfd, errfd;
+	char **eargv;
+	int eargc, infd, outfd, errfd;
 	GError *exec_err = nullptr;
 	GPid cld;
 
@@ -1814,13 +1817,13 @@ static void
 rspamc_client_cb(struct rspamd_client_connection *conn,
 				 struct rspamd_http_message *msg,
 				 const char *name, ucl_object_t *result, GString *input,
-				 gpointer ud, gdouble start_time, gdouble send_time,
+				 gpointer ud, double start_time, double send_time,
 				 const char *body, gsize bodylen,
 				 GError *err)
 {
 	struct rspamc_callback_data *cbdata = (struct rspamc_callback_data *) ud;
 	FILE *out = stdout;
-	gdouble finish = rspamd_get_ticks(FALSE), diff;
+	double finish = rspamd_get_ticks(FALSE), diff;
 
 	auto &cmd = cbdata->cmd;
 
@@ -1939,7 +1942,7 @@ rspamc_process_input(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 {
 	struct rspamd_client_connection *conn;
 	const char *p;
-	guint16 port;
+	uint16_t port;
 	GError *err = nullptr;
 	std::string hostbuf;
 
@@ -2178,6 +2181,7 @@ int main(int argc, char **argv, char **env)
 {
 	auto *kwattrs = g_queue_new();
 
+	std::locale::global(std::locale(""));
 	read_cmd_line(&argc, &argv);
 	tty = isatty(STDOUT_FILENO);
 
@@ -2290,7 +2294,7 @@ int main(int argc, char **argv, char **env)
 	add_options(kwattrs);
 	auto cmd = maybe_cmd.value();
 
-	if (start_argc == argc) {
+	if (start_argc == argc && files_list == nullptr) {
 		/* Do command without input or with stdin */
 		if (empty_input) {
 			rspamc_process_input(event_loop, cmd, nullptr, "empty", kwattrs);
@@ -2302,29 +2306,57 @@ int main(int argc, char **argv, char **env)
 	else {
 		auto cur_req = 0;
 
+		/* Process files from arguments and `files_list` */
+		std::vector<std::string> files;
+		files.reserve(argc - start_argc);
+
 		for (auto i = start_argc; i < argc; i++) {
+			files.emplace_back(argv[i]);
+		}
+
+		/* If we have list of files, read it and enrich our list */
+		if (files_list) {
+			std::ifstream in_files(files_list);
+			if (!in_files.is_open()) {
+				rspamc_print(stderr, "cannot open file {}\n", files_list);
+				exit(EXIT_FAILURE);
+			}
+			std::string line;
+			while (std::getline(in_files, line)) {
+				/* Trim spaces before inserting */
+				line.erase(0, line.find_first_not_of(" \n\r\t"));
+				line.erase(line.find_last_not_of(" \n\r\t") + 1);
+
+				/* Ignore empty lines */
+				if (!line.empty()) {
+					files.emplace_back(line);
+				}
+			}
+		}
+
+		for (const auto &file: files) {
 			if (cmd.cmd == RSPAMC_COMMAND_FUZZY_DELHASH) {
-				add_client_header(kwattrs, "Hash", argv[i]);
+				add_client_header(kwattrs, "Hash", file.c_str());
 			}
 			else {
 				struct stat st;
 
-				if (stat(argv[i], &st) == -1) {
-					rspamc_print(stderr, "cannot stat file {}\n", argv[i]);
+				if (stat(file.c_str(), &st) == -1) {
+					rspamc_print(stderr, "cannot stat file {}\n", file);
 					exit(EXIT_FAILURE);
 				}
 				if (S_ISDIR(st.st_mode)) {
 					/* Directories are processed with a separate limit */
-					rspamc_process_dir(event_loop, cmd, argv[i], kwattrs);
+					rspamc_process_dir(event_loop, cmd, file.c_str(), kwattrs);
 					cur_req = 0;
 				}
 				else {
-					in = fopen(argv[i], "r");
+					in = fopen(file.c_str(), "r");
 					if (in == nullptr) {
-						rspamc_print(stderr, "cannot open file {}\n", argv[i]);
+						rspamc_print(stderr, "cannot open file {}\n", file);
 						exit(EXIT_FAILURE);
 					}
-					rspamc_process_input(event_loop, cmd, in, argv[i], kwattrs);
+					rspamc_process_input(event_loop, cmd, in, file.c_str(), kwattrs);
 					cur_req++;
 					fclose(in);
 				}
