@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,6 +57,7 @@
 
 #include "contrib/libev/ev.h"
 #include "libstat/stat_api.h"
+#include "libserver/protocol_internal.h"
 
 struct rspamd_worker *rspamd_current_worker = NULL;
 
@@ -166,7 +167,7 @@ static void
 rspamd_worker_terminate_handlers(struct rspamd_worker *w)
 {
 	if (w->nconns == 0 &&
-		(!(w->flags & RSPAMD_WORKER_SCANNER) || w->srv->cfg->on_term_scripts == NULL)) {
+		(!(w->flags & (RSPAMD_WORKER_SCANNER | RSPAMD_WORKER_FUZZY)) || w->srv->cfg->on_term_scripts == NULL)) {
 		/*
 		 * We are here either:
 		 * - No active connections are represented
@@ -189,7 +190,7 @@ rspamd_worker_terminate_handlers(struct rspamd_worker *w)
 			if (w->state != rspamd_worker_wait_final_scripts) {
 				w->state = rspamd_worker_wait_final_scripts;
 
-				if ((w->flags & RSPAMD_WORKER_SCANNER) &&
+				if ((w->flags & (RSPAMD_WORKER_SCANNER | RSPAMD_WORKER_FUZZY)) &&
 					rspamd_worker_call_finish_handlers(w)) {
 					msg_info("performing async finishing actions");
 					w->state = rspamd_worker_wait_final_scripts;
@@ -240,6 +241,41 @@ rspamd_worker_shutdown_check(EV_P_ ev_timer *w, int revents)
 	}
 }
 
+static void
+rspamd_worker_shutdown_check_nconns(EV_P_ ev_timer *w, int revents)
+{
+	struct rspamd_worker *worker = (struct rspamd_worker *) w->data;
+
+	if (worker->state != rspamd_worker_wanna_die) {
+		if (worker->state != rspamd_worker_wait_connections) {
+			rspamd_worker_terminate_handlers(worker);
+		}
+
+		/* Check again, as rspamd_worker_terminate_handlers could change the worker's state */
+		if (worker->state == rspamd_worker_wanna_die) {
+			/* We are done, kill event loop */
+			ev_timer_stop(EV_A_ w);
+			ev_break(EV_A_ EVBREAK_ALL);
+		}
+		else {
+			/* Try again later */
+
+			if (worker->nconns > 0) {
+				ev_timer_again(EV_A_ w);
+			}
+			else {
+				/* No connections left, can close everything */
+				ev_timer_stop(EV_A_ w);
+				ev_break(EV_A_ EVBREAK_ALL);
+			}
+		}
+	}
+	else {
+		ev_timer_stop(EV_A_ w);
+		ev_break(EV_A_ EVBREAK_ALL);
+	}
+}
+
 /*
  * Config reload is designed by sending sigusr2 to active workers and pending shutdown of them
  */
@@ -326,7 +362,8 @@ rspamd_worker_term_handler(struct rspamd_worker_signal_handler *sigh, void *arg)
 									sigh->worker->srv->server_pool->tag.tagname,
 									sigh->worker->srv->server_pool->tag.uid,
 									G_STRFUNC,
-									"terminating after receiving signal %s",
+									"terminating in up to %.0f second after receiving signal %s",
+									shutdown_ts,
 									g_strsignal(sigh->signo));
 
 		rspamd_worker_stop_accept(sigh->worker);
@@ -347,6 +384,13 @@ rspamd_worker_term_handler(struct rspamd_worker_signal_handler *sigh, void *arg)
 							  0.5, 0.5);
 				ev_timer_start(sigh->event_loop, &shutdown_check_ev);
 			}
+			else {
+				/* This timer checks if we have no active connections pending and terminates once all conns are done */
+				shutdown_check_ev.data = sigh->worker;
+				ev_timer_init(&shutdown_check_ev, rspamd_worker_shutdown_check_nconns,
+							  0.5, 0.5);
+				ev_timer_start(sigh->event_loop, &shutdown_check_ev);
+			}
 		}
 		else {
 			/* Flag to die has been already set */
@@ -557,7 +601,7 @@ rspamd_controller_maybe_compress(struct rspamd_http_connection_entry *entry,
 {
 	if (entry->support_gzip) {
 		if (rspamd_fstring_gzip(&buf)) {
-			rspamd_http_message_add_header(msg, "Content-Encoding", "gzip");
+			rspamd_http_message_add_header(msg, CONTENT_ENCODING_HEADER, "gzip");
 		}
 	}
 
@@ -565,7 +609,7 @@ rspamd_controller_maybe_compress(struct rspamd_http_connection_entry *entry,
 }
 
 void rspamd_controller_send_error(struct rspamd_http_connection_entry *entry,
-								  gint code, const gchar *error_msg, ...)
+								  int code, const char *error_msg, ...)
 {
 	struct rspamd_http_message *msg;
 	va_list args;
@@ -619,7 +663,7 @@ void rspamd_controller_send_openmetrics(struct rspamd_http_connection_entry *ent
 }
 
 void rspamd_controller_send_string(struct rspamd_http_connection_entry *entry,
-								   const gchar *str)
+								   const char *str)
 {
 	struct rspamd_http_message *msg;
 	rspamd_fstring_t *reply;
@@ -680,7 +724,7 @@ rspamd_worker_drop_priv(struct rspamd_main *rspamd_main)
 	if (rspamd_main->is_privileged) {
 		if (setgid(rspamd_main->workers_gid) == -1) {
 			msg_err_main("cannot setgid to %d (%s), aborting",
-						 (gint) rspamd_main->workers_gid,
+						 (int) rspamd_main->workers_gid,
 						 strerror(errno));
 			exit(-errno);
 		}
@@ -694,7 +738,7 @@ rspamd_worker_drop_priv(struct rspamd_main *rspamd_main)
 
 		if (setuid(rspamd_main->workers_uid) == -1) {
 			msg_err_main("cannot setuid to %d (%s), aborting",
-						 (gint) rspamd_main->workers_uid,
+						 (int) rspamd_main->workers_uid,
 						 strerror(errno));
 			exit(-errno);
 		}
@@ -726,8 +770,8 @@ rspamd_worker_set_limits(struct rspamd_main *rspamd_main,
 		}
 		else {
 			msg_info_main("set max file descriptors limit: %HL cur and %HL max",
-						  (guint64) rlmt.rlim_cur,
-						  (guint64) rlmt.rlim_max);
+						  (uint64_t) rlmt.rlim_cur,
+						  (uint64_t) rlmt.rlim_max);
 		}
 	}
 	else {
@@ -739,8 +783,8 @@ rspamd_worker_set_limits(struct rspamd_main *rspamd_main,
 		}
 		else {
 			msg_info_main("use system max file descriptors limit: %HL cur and %HL max",
-						  (guint64) rlmt.rlim_cur,
-						  (guint64) rlmt.rlim_max);
+						  (uint64_t) rlmt.rlim_cur,
+						  (uint64_t) rlmt.rlim_max);
 		}
 	}
 
@@ -780,13 +824,13 @@ rspamd_worker_set_limits(struct rspamd_main *rspamd_main,
 								  "%HL was wanted, "
 								  "but we have %HL cur and %HL max",
 								  cf->rlimit_maxcore,
-								  (guint64) rlmt.rlim_cur,
-								  (guint64) rlmt.rlim_max);
+								  (uint64_t) rlmt.rlim_cur,
+								  (uint64_t) rlmt.rlim_max);
 				}
 				else {
 					msg_info_main("set max core size limit: %HL cur and %HL max",
-								  (guint64) rlmt.rlim_cur,
-								  (guint64) rlmt.rlim_max);
+								  (uint64_t) rlmt.rlim_cur,
+								  (uint64_t) rlmt.rlim_max);
 				}
 			}
 		}
@@ -799,8 +843,8 @@ rspamd_worker_set_limits(struct rspamd_main *rspamd_main,
 			}
 			else {
 				msg_info_main("use system max core size limit: %HL cur and %HL max",
-							  (guint64) rlmt.rlim_cur,
-							  (guint64) rlmt.rlim_max);
+							  (uint64_t) rlmt.rlim_cur,
+							  (uint64_t) rlmt.rlim_max);
 			}
 		}
 	}
@@ -848,13 +892,13 @@ static void
 rspamd_main_heartbeat_cb(EV_P_ ev_timer *w, int revents)
 {
 	struct rspamd_worker *wrk = (struct rspamd_worker *) w->data;
-	gdouble time_from_last = ev_time();
+	double time_from_last = ev_time();
 	struct rspamd_main *rspamd_main;
 	static struct rspamd_control_command cmd;
 	struct tm tm;
-	gchar timebuf[64];
-	gchar usec_buf[16];
-	gint r;
+	char timebuf[64];
+	char usec_buf[16];
+	int r;
 
 	time_from_last -= wrk->hb.last_event;
 	rspamd_main = wrk->srv;
@@ -866,7 +910,7 @@ rspamd_main_heartbeat_cb(EV_P_ ev_timer *w, int revents)
 		rspamd_localtime(wrk->hb.last_event, &tm);
 		r = strftime(timebuf, sizeof(timebuf), "%F %H:%M:%S", &tm);
 		rspamd_snprintf(usec_buf, sizeof(usec_buf), "%.5f",
-						wrk->hb.last_event - (gdouble) (time_t) wrk->hb.last_event);
+						wrk->hb.last_event - (double) (time_t) wrk->hb.last_event);
 		rspamd_snprintf(timebuf + r, sizeof(timebuf) - r,
 						"%s", usec_buf + 1);
 
@@ -922,7 +966,7 @@ rspamd_main_heartbeat_cb(EV_P_ ev_timer *w, int revents)
 		rspamd_localtime(wrk->hb.last_event, &tm);
 		r = strftime(timebuf, sizeof(timebuf), "%F %H:%M:%S", &tm);
 		rspamd_snprintf(usec_buf, sizeof(usec_buf), "%.5f",
-						wrk->hb.last_event - (gdouble) (time_t) wrk->hb.last_event);
+						wrk->hb.last_event - (double) (time_t) wrk->hb.last_event);
 		rspamd_snprintf(timebuf + r, sizeof(timebuf) - r,
 						"%s", usec_buf + 1);
 
@@ -968,7 +1012,7 @@ rspamd_maybe_reuseport_socket(struct rspamd_worker_listen_socket *ls)
 	}
 
 #if defined(SO_REUSEPORT) && defined(SO_REUSEADDR) && defined(LINUX)
-	gint nfd = -1;
+	int nfd = -1;
 
 	if (ls->type == RSPAMD_WORKER_SOCKET_UDP) {
 		nfd = rspamd_inet_address_listen(ls->addr,
@@ -1026,7 +1070,7 @@ rspamd_handle_child_fork(struct rspamd_worker *wrk,
 						 struct rspamd_worker_conf *cf,
 						 GHashTable *listen_sockets)
 {
-	gint rc;
+	int rc;
 	struct rlimit rlim;
 
 	/* Update pid for logging */
@@ -1245,7 +1289,7 @@ rspamd_handle_main_fork(struct rspamd_worker *wrk,
 struct rspamd_worker *
 rspamd_fork_worker(struct rspamd_main *rspamd_main,
 				   struct rspamd_worker_conf *cf,
-				   guint index,
+				   unsigned int index,
 				   struct ev_loop *ev_base,
 				   rspamd_worker_term_cb term_handler,
 				   GHashTable *listen_sockets)
@@ -1461,8 +1505,8 @@ rspamd_worker_check_controller_presence(struct rspamd_worker *w)
 
 struct rspamd_worker_session_elt {
 	void *ptr;
-	guint *pref;
-	const gchar *tag;
+	unsigned int *pref;
+	const char *tag;
 	time_t when;
 };
 
@@ -1473,7 +1517,7 @@ struct rspamd_worker_session_cache {
 	struct ev_timer periodic;
 };
 
-static gint
+static int
 rspamd_session_cache_sort_cmp(gconstpointer pa, gconstpointer pb)
 {
 	const struct rspamd_worker_session_elt
@@ -1489,12 +1533,12 @@ rspamd_sessions_cache_periodic(EV_P_ ev_timer *w, int revents)
 	struct rspamd_worker_session_cache *c =
 		(struct rspamd_worker_session_cache *) w->data;
 	GHashTableIter it;
-	gchar timebuf[32];
+	char timebuf[32];
 	gpointer k, v;
 	struct rspamd_worker_session_elt *elt;
 	struct tm tms;
 	GPtrArray *res;
-	guint i;
+	unsigned int i;
 
 	if (g_hash_table_size(c->cache) > c->cfg->max_sessions_cache) {
 		res = g_ptr_array_sized_new(g_hash_table_size(c->cache));
@@ -1505,7 +1549,7 @@ rspamd_sessions_cache_periodic(EV_P_ ev_timer *w, int revents)
 		}
 
 		msg_err("sessions cache is overflowed %d elements where %d is limit",
-				(gint) res->len, (gint) c->cfg->max_sessions_cache);
+				(int) res->len, (int) c->cfg->max_sessions_cache);
 		g_ptr_array_sort(res, rspamd_session_cache_sort_cmp);
 
 		PTR_ARRAY_FOREACH(res, i, elt)
@@ -1529,7 +1573,7 @@ rspamd_worker_session_cache_new(struct rspamd_worker *w,
 								struct ev_loop *ev_base)
 {
 	struct rspamd_worker_session_cache *c;
-	static const gdouble periodic_interval = 60.0;
+	static const double periodic_interval = 60.0;
 
 	c = g_malloc0(sizeof(*c));
 	c->ev_base = ev_base;
@@ -1545,8 +1589,8 @@ rspamd_worker_session_cache_new(struct rspamd_worker *w,
 }
 
 
-void rspamd_worker_session_cache_add(void *cache, const gchar *tag,
-									 guint *pref, void *ptr)
+void rspamd_worker_session_cache_add(void *cache, const char *tag,
+									 unsigned int *pref, void *ptr)
 {
 	struct rspamd_worker_session_cache *c = cache;
 	struct rspamd_worker_session_elt *elt;
@@ -1576,7 +1620,7 @@ rspamd_worker_monitored_on_change(struct rspamd_monitored_ctx *ctx,
 	struct rspamd_worker *worker = ud;
 	struct rspamd_config *cfg = worker->srv->cfg;
 	struct ev_loop *ev_base;
-	guchar tag[RSPAMD_MONITORED_TAG_LEN];
+	unsigned char tag[RSPAMD_MONITORED_TAG_LEN];
 	static struct rspamd_srv_command srv_cmd;
 
 	rspamd_monitored_get_tag(m, tag);
@@ -1721,10 +1765,10 @@ rspamd_enable_accept_event(EV_P_ ev_timer *w, int revents)
 	ev_io_start(EV_A_ & ac_ev->accept_ev);
 }
 
-void rspamd_worker_throttle_accept_events(gint sock, void *data)
+void rspamd_worker_throttle_accept_events(int sock, void *data)
 {
 	struct rspamd_worker_accept_event *head, *cur;
-	const gdouble throttling = 0.5;
+	const double throttling = 0.5;
 
 	head = (struct rspamd_worker_accept_event *) data;
 
@@ -1808,8 +1852,8 @@ rspamd_check_termination_clause(struct rspamd_main *rspamd_main,
 					WEXITSTATUS(res),
 					g_strsignal(WTERMSIG(res)),
 					wrk->cores_throttled ? "yes" : "no",
-					(gint64) rlmt.rlim_cur,
-					(gint64) rlmt.rlim_max);
+					(int64_t) rlmt.rlim_cur,
+					(int64_t) rlmt.rlim_max);
 #else
 				msg_warn_main(
 					"%s process %P terminated abnormally with exit code %d by signal: %s"
@@ -1853,8 +1897,8 @@ rspamd_check_termination_clause(struct rspamd_main *rspamd_main,
 #ifdef WITH_HYPERSCAN
 gboolean
 rspamd_worker_hyperscan_ready(struct rspamd_main *rspamd_main,
-							  struct rspamd_worker *worker, gint fd,
-							  gint attached_fd,
+							  struct rspamd_worker *worker, int fd,
+							  int attached_fd,
 							  struct rspamd_control_command *cmd,
 							  gpointer ud)
 {
@@ -1884,7 +1928,7 @@ rspamd_worker_hyperscan_ready(struct rspamd_main *rspamd_main,
 #endif /* With Hyperscan */
 
 gboolean
-rspamd_worker_check_context(gpointer ctx, guint64 magic)
+rspamd_worker_check_context(gpointer ctx, uint64_t magic)
 {
 	struct rspamd_abstract_worker_ctx *actx = (struct rspamd_abstract_worker_ctx *) ctx;
 
@@ -1893,8 +1937,8 @@ rspamd_worker_check_context(gpointer ctx, guint64 magic)
 
 static gboolean
 rspamd_worker_log_pipe_handler(struct rspamd_main *rspamd_main,
-							   struct rspamd_worker *worker, gint fd,
-							   gint attached_fd,
+							   struct rspamd_worker *worker, int fd,
+							   int attached_fd,
 							   struct rspamd_control_command *cmd,
 							   gpointer ud)
 {
@@ -1928,8 +1972,8 @@ rspamd_worker_log_pipe_handler(struct rspamd_main *rspamd_main,
 
 static gboolean
 rspamd_worker_monitored_handler(struct rspamd_main *rspamd_main,
-								struct rspamd_worker *worker, gint fd,
-								gint attached_fd,
+								struct rspamd_worker *worker, int fd,
+								int attached_fd,
 								struct rspamd_control_command *cmd,
 								gpointer ud)
 {
@@ -1996,9 +2040,9 @@ void rspamd_controller_store_saved_stats(struct rspamd_main *rspamd_main,
 	struct rspamd_stat *stat;
 	ucl_object_t *top, *sub;
 	struct ucl_emitter_functions *efuncs;
-	gint i, fd;
+	int i, fd;
 	FILE *fp;
-	gchar fpath[PATH_MAX];
+	char fpath[PATH_MAX];
 
 	if (cfg->stats_file == NULL) {
 		return;
@@ -2082,7 +2126,7 @@ rspamd_controller_load_saved_stats(struct rspamd_main *rspamd_main,
 	ucl_object_t *obj;
 	const ucl_object_t *elt, *subelt;
 	struct rspamd_stat *stat, stat_copy;
-	gint i;
+	int i;
 
 	if (cfg->stats_file == NULL) {
 		return;
@@ -2164,9 +2208,9 @@ rspamd_controller_rrd_update(EV_P_ ev_timer *w, int revents)
 		(struct rspamd_controller_periodics_cbdata *) w->data;
 	struct rspamd_stat *stat;
 	GArray ar;
-	gdouble points[METRIC_ACTION_MAX];
+	double points[METRIC_ACTION_MAX];
 	GError *err = NULL;
-	guint i;
+	unsigned int i;
 
 	g_assert(cbd->rrd != NULL);
 	stat = cbd->stat;
@@ -2175,7 +2219,7 @@ rspamd_controller_rrd_update(EV_P_ ev_timer *w, int revents)
 		points[i] = stat->actions_stat[i];
 	}
 
-	ar.data = (gchar *) points;
+	ar.data = (char *) points;
 	ar.len = sizeof(points);
 
 	if (!rspamd_rrd_add_record(cbd->rrd, &ar, rspamd_get_calendar_ticks(),
@@ -2265,8 +2309,8 @@ void rspamd_worker_init_controller(struct rspamd_worker *worker,
 	}
 }
 
-gdouble
-rspamd_worker_check_and_adjust_timeout(struct rspamd_config *cfg, gdouble timeout)
+double
+rspamd_worker_check_and_adjust_timeout(struct rspamd_config *cfg, double timeout)
 {
 	if (isnan(timeout)) {
 		/* Use implicit timeout from cfg->task_timeout */
@@ -2311,3 +2355,205 @@ rspamd_worker_check_and_adjust_timeout(struct rspamd_config *cfg, gdouble timeou
 	/* TODO: maybe adjust timeout */
 	return timeout;
 }
+
+ucl_object_t *
+rspamd_worker_metrics_object(struct rspamd_config *cfg, struct rspamd_stat *stat, ev_tstamp uptime)
+{
+	rspamd_mempool_stat_t mem_st;
+	memset(&mem_st, 0, sizeof(mem_st));
+	rspamd_mempool_stat(&mem_st);
+
+	ucl_object_t *top = ucl_object_typed_new(UCL_OBJECT);
+
+	ucl_object_insert_key(top, ucl_object_fromstring(RVERSION), "version", 0, false);
+	ucl_object_insert_key(top, ucl_object_fromstring(cfg->checksum), "config_id", 0, false);
+	ucl_object_insert_key(top, ucl_object_fromdouble(uptime), "uptime", 0, false);
+	ucl_object_insert_key(top, ucl_object_fromint(stat->messages_scanned), "scanned", 0, false);
+	ucl_object_insert_key(top, ucl_object_fromint(stat->messages_learned), "learned", 0, false);
+
+	gsize cnt = MAX_AVG_TIME_SLOTS;
+	float sum = rspamd_sum_floats(stat->avg_time.avg_time, &cnt);
+
+	ucl_object_insert_key(top,
+						  ucl_object_fromdouble(cnt > 0 ? (double) sum / cnt : 0.0), "avg_scan_time", 0, false);
+
+	unsigned spam = 0, ham = 0;
+
+	if (stat->messages_scanned > 0) {
+		ucl_object_t *sub = ucl_object_typed_new(UCL_OBJECT);
+		for (int i = METRIC_ACTION_REJECT; i <= METRIC_ACTION_NOACTION; i++) {
+			ucl_object_insert_key(sub,
+								  ucl_object_fromint(stat->actions_stat[i]),
+								  rspamd_action_to_str(i), 0, false);
+			if (i < METRIC_ACTION_GREYLIST) {
+				spam += stat->actions_stat[i];
+			}
+			else {
+				ham += stat->actions_stat[i];
+			}
+		}
+		ucl_object_insert_key(top, sub, "actions", 0, false);
+	}
+
+	ucl_object_insert_key(top, ucl_object_fromint(spam), "spam_count", 0, false);
+	ucl_object_insert_key(top, ucl_object_fromint(ham), "ham_count", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(stat->connections_count), "connections", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(stat->control_connections_count),
+						  "control_connections", 0, false);
+
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.pools_allocated), "pools_allocated", 0,
+						  false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.pools_freed), "pools_freed", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.bytes_allocated), "bytes_allocated", 0,
+						  false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(
+							  mem_st.chunks_allocated),
+						  "chunks_allocated", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.shared_chunks_allocated),
+						  "shared_chunks_allocated", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.chunks_freed), "chunks_freed", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(
+							  mem_st.oversized_chunks),
+						  "chunks_oversized", 0, false);
+	ucl_object_insert_key(top,
+						  ucl_object_fromint(mem_st.fragmented_size), "fragmented", 0, false);
+
+	return top;
+}
+
+rspamd_fstring_t *
+rspamd_metrics_to_prometheus_string(const ucl_object_t *top)
+{
+	rspamd_fstring_t *output = rspamd_fstring_sized_new(1024);
+
+	rspamd_printf_fstring(&output, "# HELP rspamd_build_info A metric with a constant '1' value "
+								   "labeled by version from which rspamd was built.\n");
+	rspamd_printf_fstring(&output, "# TYPE rspamd_build_info gauge\n");
+	rspamd_printf_fstring(&output, "rspamd_build_info{version=\"%s\"} 1\n",
+						  ucl_object_tostring(ucl_object_lookup(top, "version")));
+	rspamd_printf_fstring(&output, "# HELP rspamd_config A metric with a constant '1' value "
+								   "labeled by id of the current config.\n");
+	rspamd_printf_fstring(&output, "# TYPE rspamd_config gauge\n");
+	rspamd_printf_fstring(&output, "rspamd_config{id=\"%s\"} 1\n",
+						  ucl_object_tostring(ucl_object_lookup(top, "config_id")));
+
+
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_scan_time_average",
+							   "gauge",
+							   "Average messages scan time.",
+							   "avg_scan_time");
+	rspamd_metrics_add_integer(&output, top,
+							   "process_start_time_seconds",
+							   "gauge",
+							   "Start time of the process since unix epoch in seconds.",
+							   "start_time");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_read_only",
+							   "gauge",
+							   "Whether the rspamd instance is read-only.",
+							   "read_only");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_scanned_total",
+							   "counter",
+							   "Scanned messages.",
+							   "scanned");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_learned_total",
+							   "counter",
+							   "Learned messages.",
+							   "learned");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_spam_total",
+							   "counter",
+							   "Messages classified as spam.",
+							   "spam_count");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_ham_total",
+							   "counter",
+							   "Messages classified as ham.",
+							   "ham_count");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_connections",
+							   "gauge",
+							   "Active connections.",
+							   "connections");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_control_connections_total",
+							   "gauge",
+							   "Control connections.",
+							   "control_connections");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_pools_allocated",
+							   "gauge",
+							   "Pools allocated.",
+							   "pools_allocated");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_pools_freed",
+							   "gauge",
+							   "Pools freed.",
+							   "pools_freed");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_allocated_bytes",
+							   "gauge",
+							   "Bytes allocated.",
+							   "bytes_allocated");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_chunks_allocated",
+							   "gauge",
+							   "Memory pools: current chunks allocated.",
+							   "chunks_allocated");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_shared_chunks_allocated",
+							   "gauge",
+							   "Memory pools: current shared chunks allocated.",
+							   "shared_chunks_allocated");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_chunks_freed",
+							   "gauge",
+							   "Memory pools: current chunks freed.",
+							   "chunks_freed");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_chunks_oversized",
+							   "gauge",
+							   "Memory pools: current chunks oversized (needs extra allocation/fragmentation).",
+							   "chunks_oversized");
+	rspamd_metrics_add_integer(&output, top,
+							   "rspamd_fragmented",
+							   "gauge",
+							   "Memory pools: fragmented memory waste.",
+							   "fragmented");
+
+	const ucl_object_t *acts_obj = ucl_object_lookup(top, "actions");
+
+	if (acts_obj) {
+		rspamd_printf_fstring(&output, "# HELP rspamd_actions_total Actions labelled by action type.\n");
+		rspamd_printf_fstring(&output, "# TYPE rspamd_actions_total counter\n");
+		for (int i = METRIC_ACTION_REJECT; i <= METRIC_ACTION_NOACTION; i++) {
+			const char *str_act = rspamd_action_to_str(i);
+			const ucl_object_t *act = ucl_object_lookup(acts_obj, str_act);
+
+			if (act) {
+				rspamd_printf_fstring(&output, "rspamd_actions_total{type=\"%s\"} %L\n",
+									  str_act,
+									  ucl_object_toint(act));
+			}
+			else {
+				rspamd_printf_fstring(&output, "rspamd_actions_total{type=\"%s\"} 0\n",
+									  str_act);
+			}
+		}
+	}
+
+	/* Must be finalized and freed by caller */
+	return output;
+}
\ No newline at end of file
