@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -27,19 +27,17 @@
 #include <sys/wait.h>
 #endif
 
-static gboolean openssl = FALSE;
 static gboolean verify = FALSE;
 static gboolean quiet = FALSE;
-static gchar *suffix = NULL;
-static gchar *pubkey_file = NULL;
-static gchar *pubkey = NULL;
-static gchar *pubout = NULL;
-static gchar *keypair_file = NULL;
-static gchar *editor = NULL;
+static char *suffix = NULL;
+static char *pubkey_file = NULL;
+static char *pubkey = NULL;
+static char *pubout = NULL;
+static char *keypair_file = NULL;
+static char *editor = NULL;
 static gboolean edit = FALSE;
-enum rspamd_cryptobox_mode mode = RSPAMD_CRYPTOBOX_MODE_25519;
 
-static void rspamadm_signtool(gint argc, gchar **argv,
+static void rspamadm_signtool(int argc, char **argv,
 							  const struct rspamadm_command *cmd);
 static const char *rspamadm_signtool_help(gboolean full_help,
 										  const struct rspamadm_command *cmd);
@@ -53,8 +51,6 @@ struct rspamadm_command signtool_command = {
 };
 
 static GOptionEntry entries[] = {
-	{"openssl", 'o', 0, G_OPTION_ARG_NONE, &openssl,
-	 "Generate openssl nistp256 keypair not curve25519 one", NULL},
 	{"verify", 'v', 0, G_OPTION_ARG_NONE, &verify,
 	 "Verify signatures and not sign", NULL},
 	{"suffix", 'S', 0, G_OPTION_ARG_STRING, &suffix,
@@ -103,15 +99,15 @@ rspamadm_signtool_help(gboolean full_help,
 	return help_str;
 }
 
-static gint
-rspamadm_edit_file(const gchar *fname)
+static int
+rspamadm_edit_file(const char *fname)
 {
-	gchar tmppath[PATH_MAX], run_cmdline[PATH_MAX];
-	guchar *map;
+	char tmppath[PATH_MAX], run_cmdline[PATH_MAX];
+	unsigned char *map;
 	gsize len = 0;
-	gint fd_out, retcode, child_argc;
+	int fd_out, retcode, child_argc;
 	GPid child_pid;
-	gchar *tmpdir, **child_argv = NULL;
+	char *tmpdir, **child_argv = NULL;
 	struct stat st;
 	GError *err = NULL;
 
@@ -278,14 +274,14 @@ rspamadm_edit_file(const gchar *fname)
 }
 
 static bool
-rspamadm_sign_file(const gchar *fname, struct rspamd_cryptobox_keypair *kp)
+rspamadm_sign_file(const char *fname, struct rspamd_cryptobox_keypair *kp)
 {
-	gint fd_sig, fd_input;
-	guchar sig[rspamd_cryptobox_MAX_SIGBYTES], *map;
-	gchar sigpath[PATH_MAX];
+	int fd_sig, fd_input;
+	unsigned char sig[rspamd_cryptobox_MAX_SIGBYTES], *map;
+	char sigpath[PATH_MAX];
 	FILE *pub_fp;
 	struct stat st;
-	const guchar *sk;
+	const unsigned char *sk;
 
 	if (suffix == NULL) {
 		suffix = ".sig";
@@ -327,11 +323,8 @@ rspamadm_sign_file(const gchar *fname, struct rspamd_cryptobox_keypair *kp)
 		exit(EXIT_FAILURE);
 	}
 
-	g_assert(rspamd_cryptobox_MAX_SIGBYTES >=
-			 rspamd_cryptobox_signature_bytes(mode));
-
 	sk = rspamd_keypair_component(kp, RSPAMD_KEYPAIR_COMPONENT_SK, NULL);
-	rspamd_cryptobox_sign(sig, NULL, map, st.st_size, sk, mode);
+	rspamd_cryptobox_sign(sig, NULL, map, st.st_size, sk);
 
 	if (edit) {
 		/* We also need to rename .new file */
@@ -348,7 +341,7 @@ rspamadm_sign_file(const gchar *fname, struct rspamd_cryptobox_keypair *kp)
 
 	rspamd_snprintf(sigpath, sizeof(sigpath), "%s%s", fname, suffix);
 
-	if (write(fd_sig, sig, rspamd_cryptobox_signature_bytes(mode)) == -1) {
+	if (write(fd_sig, sig, crypto_sign_bytes()) == -1) {
 		rspamd_fprintf(stderr, "cannot write signature to %s: %s\n", sigpath,
 					   strerror(errno));
 		exit(EXIT_FAILURE);
@@ -373,7 +366,8 @@ rspamadm_sign_file(const gchar *fname, struct rspamd_cryptobox_keypair *kp)
 		}
 		else {
 			b32_pk = rspamd_keypair_print(kp,
-										  RSPAMD_KEYPAIR_PUBKEY | RSPAMD_KEYPAIR_BASE32);
+										  RSPAMD_KEYPAIR_ENCODING_DEFAULT,
+										  RSPAMD_KEYPAIR_PUBKEY);
 
 			if (b32_pk) {
 				rspamd_fprintf(pub_fp, "%v", b32_pk);
@@ -392,17 +386,14 @@ rspamadm_sign_file(const gchar *fname, struct rspamd_cryptobox_keypair *kp)
 }
 
 static bool
-rspamadm_verify_file(const gchar *fname, const guchar *pk)
+rspamadm_verify_file(const char *fname, const unsigned char *pk)
 {
-	gint fd_sig, fd_input;
-	guchar *map, *map_sig;
-	gchar sigpath[PATH_MAX];
+	int fd_sig, fd_input;
+	unsigned char *map, *map_sig;
+	char sigpath[PATH_MAX];
 	struct stat st, st_sig;
 	bool ret;
 
-	g_assert(rspamd_cryptobox_MAX_SIGBYTES >=
-			 rspamd_cryptobox_signature_bytes(mode));
-
 	if (suffix == NULL) {
 		suffix = ".sig";
 	}
@@ -439,10 +430,10 @@ rspamadm_verify_file(const gchar *fname, const guchar *pk)
 
 	g_assert(fstat(fd_sig, &st_sig) != -1);
 
-	if (st_sig.st_size != rspamd_cryptobox_signature_bytes(mode)) {
+	if (st_sig.st_size != crypto_sign_bytes()) {
 		close(fd_sig);
 		rspamd_fprintf(stderr, "invalid signature size %s: %ud\n", fname,
-					   (guint) st_sig.st_size);
+					   (unsigned int) st_sig.st_size);
 		munmap(map, st.st_size);
 		exit(EXIT_FAILURE);
 	}
@@ -458,7 +449,7 @@ rspamadm_verify_file(const gchar *fname, const guchar *pk)
 	}
 
 	ret = rspamd_cryptobox_verify(map_sig, st_sig.st_size,
-								  map, st.st_size, pk, mode);
+								  map, st.st_size, pk);
 	munmap(map, st.st_size);
 	munmap(map_sig, st_sig.st_size);
 
@@ -476,7 +467,7 @@ rspamadm_verify_file(const gchar *fname, const guchar *pk)
 
 
 static void
-rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
+rspamadm_signtool(int argc, char **argv, const struct rspamadm_command *cmd)
 {
 	GOptionContext *context;
 	GError *error = NULL;
@@ -485,7 +476,7 @@ rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 	struct rspamd_cryptobox_pubkey *pk;
 	struct rspamd_cryptobox_keypair *kp;
 	gsize fsize, flen;
-	gint i;
+	int i;
 
 	context = g_option_context_new(
 		"keypair - create encryption keys");
@@ -503,10 +494,6 @@ rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 
 	g_option_context_free(context);
 
-	if (openssl) {
-		mode = RSPAMD_CRYPTOBOX_MODE_NIST;
-	}
-
 	if (verify && (!pubkey && !pubkey_file)) {
 		rspamd_fprintf(stderr, "no pubkey for verification\n");
 		exit(EXIT_FAILURE);
@@ -520,8 +507,8 @@ rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 		g_assert(pubkey || pubkey_file);
 
 		if (pubkey_file) {
-			gint fd;
-			gchar *map;
+			int fd;
+			char *map;
 			struct stat st;
 
 			fd = open(pubkey_file, O_RDONLY);
@@ -549,14 +536,13 @@ rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 				flen--;
 			}
 
-			pk = rspamd_pubkey_from_base32(map, flen,
-										   RSPAMD_KEYPAIR_SIGN, mode);
+			pk = rspamd_pubkey_from_base32(map, flen, RSPAMD_KEYPAIR_SIGN);
 
 			if (pk == NULL) {
 				rspamd_fprintf(stderr, "bad size %s: %ud, %ud expected\n",
 							   pubkey_file,
-							   (guint) flen,
-							   rspamd_cryptobox_pk_sig_bytes(mode));
+							   (unsigned int) flen,
+							   crypto_sign_publickeybytes());
 				exit(EXIT_FAILURE);
 			}
 
@@ -564,13 +550,13 @@ rspamadm_signtool(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 		}
 		else {
 			pk = rspamd_pubkey_from_base32(pubkey, strlen(pubkey),
-										   RSPAMD_KEYPAIR_SIGN, mode);
+										   RSPAMD_KEYPAIR_SIGN);
 
 			if (pk == NULL) {
 				rspamd_fprintf(stderr, "bad size %s: %ud, %ud expected\n",
 							   pubkey_file,
-							   (guint) strlen(pubkey),
-							   rspamd_cryptobox_pk_sig_bytes(mode));
+							   (unsigned int) strlen(pubkey),
+							   crypto_sign_publickeybytes());
 				exit(EXIT_FAILURE);
 			}
 		}
