@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -24,10 +24,9 @@
 static const int mapping_size = 64 * 8192 + 1;
 static const int max_seg = 32;
 static const int random_fuzz_cnt = 10000;
-enum rspamd_cryptobox_mode mode = RSPAMD_CRYPTOBOX_MODE_25519;
 
 static void *
-create_mapping(int mapping_len, guchar **beg, guchar **end)
+create_mapping(int mapping_len, unsigned char **beg, unsigned char **end)
 {
 	void *map;
 	int psize = getpagesize();
@@ -38,7 +37,7 @@ create_mapping(int mapping_len, guchar **beg, guchar **end)
 	memset(map, 0, mapping_len + psize * 3);
 	mprotect(map, psize, PROT_NONE);
 	/* Misalign pointer */
-	*beg = ((guchar *) map) + psize + 1;
+	*beg = ((unsigned char *) map) + psize + 1;
 	*end = *beg + mapping_len;
 	mprotect(*beg + mapping_len - 1 + psize, psize, PROT_NONE);
 
@@ -47,14 +46,14 @@ create_mapping(int mapping_len, guchar **beg, guchar **end)
 
 static void
 check_result(const rspamd_nm_t key, const rspamd_nonce_t nonce,
-			 const rspamd_mac_t mac, guchar *begin, guchar *end)
+			 const rspamd_mac_t mac, unsigned char *begin, unsigned char *end)
 {
-	guint64 *t = (guint64 *) begin;
+	uint64_t *t = (uint64_t *) begin;
 
 	g_assert(rspamd_cryptobox_decrypt_nm_inplace(begin, end - begin, nonce, key,
-												 mac, mode));
+												 mac));
 
-	while (t < (guint64 *) end) {
+	while (t < (uint64_t *) end) {
 		g_assert(*t == 0);
 		t++;
 	}
@@ -62,10 +61,10 @@ check_result(const rspamd_nm_t key, const rspamd_nonce_t nonce,
 
 static int
 create_random_split(struct rspamd_cryptobox_segment *seg, int mseg,
-					guchar *begin, guchar *end)
+					unsigned char *begin, unsigned char *end)
 {
 	gsize remain = end - begin;
-	gint used = 0;
+	int used = 0;
 
 	while (remain > 0 && used < mseg - 1) {
 		seg->data = begin;
@@ -88,10 +87,10 @@ create_random_split(struct rspamd_cryptobox_segment *seg, int mseg,
 
 static int
 create_realistic_split(struct rspamd_cryptobox_segment *seg, int mseg,
-					   guchar *begin, guchar *end)
+					   unsigned char *begin, unsigned char *end)
 {
 	gsize remain = end - begin;
-	gint used = 0;
+	int used = 0;
 	static const int small_seg = 512, medium_seg = 2048;
 
 	while (remain > 0 && used < mseg - 1) {
@@ -126,10 +125,10 @@ create_realistic_split(struct rspamd_cryptobox_segment *seg, int mseg,
 static int
 create_constrained_split(struct rspamd_cryptobox_segment *seg, int mseg,
 						 int constraint,
-						 guchar *begin, guchar *end)
+						 unsigned char *begin, unsigned char *end)
 {
 	gsize remain = end - begin;
-	gint used = 0;
+	int used = 0;
 
 	while (remain > 0 && used < mseg - 1) {
 		seg->data = begin;
@@ -155,13 +154,13 @@ create_constrained_split(struct rspamd_cryptobox_segment *seg, int mseg,
 void rspamd_cryptobox_test_func(void)
 {
 	void *map;
-	guchar *begin, *end;
+	unsigned char *begin, *end;
 	rspamd_nm_t key;
 	rspamd_nonce_t nonce;
 	rspamd_mac_t mac;
 	struct rspamd_cryptobox_segment *seg;
 	double t1, t2;
-	gint i, cnt, ms;
+	int i, cnt, ms;
 	gboolean checked_openssl = FALSE;
 
 	map = create_mapping(mapping_size, &begin, &end);
@@ -174,33 +173,17 @@ void rspamd_cryptobox_test_func(void)
 
 	/* Test baseline */
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encrypt_nm_inplace(begin, end - begin, nonce, key, mac,
-										mode);
+	rspamd_cryptobox_encrypt_nm_inplace(begin, end - begin, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 	check_result(key, nonce, mac, begin, end);
 
 	msg_info("baseline encryption: %.0f", t2 - t1);
 
-	mode = RSPAMD_CRYPTOBOX_MODE_NIST;
-	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encrypt_nm_inplace(begin,
-										end - begin,
-										nonce,
-										key,
-										mac,
-										mode);
-	t2 = rspamd_get_ticks(TRUE);
-	check_result(key, nonce, mac, begin, end);
-
-	msg_info("openssl baseline encryption: %.0f", t2 - t1);
-	mode = RSPAMD_CRYPTOBOX_MODE_25519;
-
-start:
 	/* A single chunk as vector */
 	seg[0].data = begin;
 	seg[0].len = end - begin;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 1, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 1, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -213,7 +196,7 @@ start:
 	seg[1].data = begin + seg[0].len;
 	seg[1].len = (end - begin) - seg[0].len;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -225,7 +208,7 @@ start:
 	seg[1].data = begin + seg[0].len;
 	seg[1].len = (end - begin) - seg[0].len;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -237,7 +220,7 @@ start:
 	seg[1].data = begin + seg[0].len;
 	seg[1].len = (end - begin) - seg[0].len;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -250,7 +233,7 @@ start:
 	seg[1].data = begin + seg[0].len;
 	seg[1].len = (end - begin) - seg[0].len;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 2, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -265,7 +248,7 @@ start:
 	seg[2].data = begin + seg[0].len + seg[1].len;
 	seg[2].len = (end - begin) - seg[0].len - seg[1].len;
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, 3, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, 3, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -274,7 +257,7 @@ start:
 
 	cnt = create_random_split(seg, max_seg, begin, end);
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -283,7 +266,7 @@ start:
 
 	cnt = create_realistic_split(seg, max_seg, begin, end);
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -292,7 +275,7 @@ start:
 
 	cnt = create_constrained_split(seg, max_seg + 1, 32, begin, end);
 	t1 = rspamd_get_ticks(TRUE);
-	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+	rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 	t2 = rspamd_get_ticks(TRUE);
 
 	check_result(key, nonce, mac, begin, end);
@@ -303,7 +286,7 @@ start:
 		ms = ottery_rand_range(i % max_seg * 2) + 1;
 		cnt = create_random_split(seg, ms, begin, end);
 		t1 = rspamd_get_ticks(TRUE);
-		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 		t2 = rspamd_get_ticks(TRUE);
 
 		check_result(key, nonce, mac, begin, end);
@@ -316,7 +299,7 @@ start:
 		ms = ottery_rand_range(i % max_seg * 2) + 1;
 		cnt = create_realistic_split(seg, ms, begin, end);
 		t1 = rspamd_get_ticks(TRUE);
-		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 		t2 = rspamd_get_ticks(TRUE);
 
 		check_result(key, nonce, mac, begin, end);
@@ -329,7 +312,7 @@ start:
 		ms = ottery_rand_range(i % max_seg * 10) + 1;
 		cnt = create_constrained_split(seg, ms, i, begin, end);
 		t1 = rspamd_get_ticks(TRUE);
-		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac, mode);
+		rspamd_cryptobox_encryptv_nm_inplace(seg, cnt, nonce, key, mac);
 		t2 = rspamd_get_ticks(TRUE);
 
 		check_result(key, nonce, mac, begin, end);
@@ -338,10 +321,4 @@ start:
 			msg_info("constrained fuzz iterations: %d", i);
 		}
 	}
-
-	if (!checked_openssl) {
-		checked_openssl = TRUE;
-		mode = RSPAMD_CRYPTOBOX_MODE_NIST;
-		goto start;
-	}
 }
